@@ -50,7 +50,7 @@ from .common import (
     mock_data_received,
     send_ping_request,
     send_ping_response,
-    send_plaintext_connect_response,
+    send_plaintext_auth_response,
     send_plaintext_hello,
     utcnow,
 )
@@ -94,7 +94,7 @@ async def test_connect_hello_login_with_login_false_and_password_none(
         # Wait for connection to be established
         await connected.wait()
 
-        # Send only HelloResponse (no ConnectResponse since login=False)
+        # Send only HelloResponse (no AuthenticationResponse since login=False)
         assert protocol is not None
         hello_response = bytes.fromhex(
             "003602080110091a216d6173746572617672656c61792028657"
@@ -104,7 +104,7 @@ async def test_connect_hello_login_with_login_false_and_password_none(
         mock_data_received(protocol, hello_response)
 
         # The connection should complete successfully without trying to pop
-        # a ConnectResponse from an empty list
+        # a AuthenticationResponse from an empty list
         await connect_task
 
         assert conn.is_connected
@@ -117,7 +117,7 @@ async def test_connect(
     ],
 ) -> None:
     """Test that a plaintext connection works."""
-    conn, transport, protocol, connect_task = plaintext_connect_task_no_login
+    conn, _transport, protocol, connect_task = plaintext_connect_task_no_login
     mock_data_received(
         protocol,
         bytes.fromhex(
@@ -244,7 +244,7 @@ async def test_plaintext_connection(
 ) -> None:
     """Test that a plaintext connection works."""
     messages = []
-    conn, transport, protocol, connect_task = plaintext_connect_task_no_login
+    conn, _transport, protocol, connect_task = plaintext_connect_task_no_login
 
     def on_msg(msg):
         messages.append(msg)
@@ -488,7 +488,7 @@ async def test_finish_connection_times_out(
     caplog: pytest.LogCaptureFixture,
 ) -> None:
     """Test handling of finish connection timing out."""
-    conn, transport, protocol, connect_task = plaintext_connect_task_no_login
+    conn, _transport, protocol, connect_task = plaintext_connect_task_no_login
     messages = []
 
     def on_msg(msg):
@@ -505,7 +505,7 @@ async def test_finish_connection_times_out(
     await asyncio.sleep(0)
 
     with pytest.raises(
-        APIConnectionError, match="Timeout waiting for HelloResponse after 30.0s"
+        APIConnectionError, match=r"Timeout waiting for HelloResponse after 30.0s"
     ):
         await connect_task
 
@@ -613,31 +613,15 @@ async def test_plaintext_connection_fails_handshake(
     await asyncio.sleep(0)
 
 
-async def test_connect_wrong_password(
-    plaintext_connect_task_with_login: tuple[
-        APIConnection, asyncio.Transport, APIPlaintextFrameHelper, asyncio.Task
-    ],
-) -> None:
-    conn, transport, protocol, connect_task = plaintext_connect_task_with_login
-
-    send_plaintext_hello(protocol)
-    send_plaintext_connect_response(protocol, True)
-
-    with pytest.raises(InvalidAuthAPIError):
-        await connect_task
-
-    assert not conn.is_connected
-
-
 async def test_connect_correct_password(
     plaintext_connect_task_with_login: tuple[
         APIConnection, asyncio.Transport, APIPlaintextFrameHelper, asyncio.Task
     ],
 ) -> None:
-    conn, transport, protocol, connect_task = plaintext_connect_task_with_login
+    conn, _transport, protocol, connect_task = plaintext_connect_task_with_login
 
     send_plaintext_hello(protocol)
-    send_plaintext_connect_response(protocol, False)
+    send_plaintext_auth_response(protocol, False)
 
     await connect_task
 
@@ -668,13 +652,13 @@ async def test_connect_password_required_but_not_sent(
         # Send hello response
         send_plaintext_hello(protocol)
 
-        # Complete the connection first since we're not expecting a ConnectResponse
+        # Complete the connection first since we're not expecting a AuthenticationResponse
         await connect_task
         assert conn.is_connected
 
-        # Device sends ConnectResponse with invalid_password=True
+        # Device sends AuthenticationResponse with invalid_password=True
         # This simulates a device that requires a password but we didn't send one
-        send_plaintext_connect_response(protocol, True)
+        send_plaintext_auth_response(protocol, True)
 
         # Give the event loop a chance to process the message
         await asyncio.sleep(0)
@@ -691,10 +675,10 @@ async def test_connect_wrong_version(
         APIConnection, asyncio.Transport, APIPlaintextFrameHelper, asyncio.Task
     ],
 ) -> None:
-    conn, transport, protocol, connect_task = plaintext_connect_task_with_login
+    conn, _transport, protocol, connect_task = plaintext_connect_task_with_login
 
     send_plaintext_hello(protocol, 3, 2)
-    send_plaintext_connect_response(protocol, False)
+    send_plaintext_auth_response(protocol, False)
 
     with pytest.raises(APIConnectionError, match="Incompatible API version"):
         await connect_task
@@ -707,9 +691,9 @@ async def test_connect_wrong_name(
         APIConnection, asyncio.Transport, APIPlaintextFrameHelper, asyncio.Task
     ],
 ) -> None:
-    conn, transport, protocol, connect_task = plaintext_connect_task_expected_name
+    conn, _transport, protocol, connect_task = plaintext_connect_task_expected_name
     send_plaintext_hello(protocol)
-    send_plaintext_connect_response(protocol, False)
+    send_plaintext_auth_response(protocol, False)
 
     with pytest.raises(
         APIConnectionError,
@@ -726,10 +710,10 @@ async def test_force_disconnect_fails(
         APIConnection, asyncio.Transport, APIPlaintextFrameHelper, asyncio.Task
     ],
 ) -> None:
-    conn, transport, protocol, connect_task = plaintext_connect_task_with_login
+    conn, _transport, protocol, connect_task = plaintext_connect_task_with_login
 
     send_plaintext_hello(protocol)
-    send_plaintext_connect_response(protocol, False)
+    send_plaintext_auth_response(protocol, False)
 
     await connect_task
     assert conn.is_connected
@@ -754,7 +738,7 @@ async def test_connection_lost_while_connecting(
     ],
     exception_map: tuple[Exception, Exception],
 ) -> None:
-    conn, transport, protocol, connect_task = plaintext_connect_task_with_login
+    conn, _transport, protocol, connect_task = plaintext_connect_task_with_login
 
     exception, raised_exception = exception_map
     protocol.connection_lost(exception)
@@ -862,7 +846,7 @@ async def test_connect_resolver_times_out(
         ),
         pytest.raises(
             ResolveAPIError,
-            match="Timeout while resolving IP address for fake.address",
+            match=r"Timeout while resolving IP address for fake.address",
         ),
     ):
         await connect(conn, login=False)
@@ -902,7 +886,7 @@ async def test_disconnect_fails_to_send_response(
         transport.reset_mock()
 
     send_plaintext_hello(protocol)
-    send_plaintext_connect_response(protocol, False)
+    send_plaintext_auth_response(protocol, False)
 
     await connect_task
     assert client._connection.is_connected
@@ -950,7 +934,7 @@ async def test_disconnect_success_case(
         transport.reset_mock()
 
     send_plaintext_hello(protocol)
-    send_plaintext_connect_response(protocol, False)
+    send_plaintext_auth_response(protocol, False)
 
     await connect_task
     assert client._connection.is_connected
@@ -972,7 +956,7 @@ async def test_ping_disconnects_after_no_responses(
     conn, transport, protocol, connect_task = plaintext_connect_task_with_login
 
     send_plaintext_hello(protocol)
-    send_plaintext_connect_response(protocol, False)
+    send_plaintext_auth_response(protocol, False)
 
     await connect_task
 
@@ -1011,7 +995,7 @@ async def test_ping_does_not_disconnect_if_we_get_responses(
     conn, transport, protocol, connect_task = plaintext_connect_task_with_login
 
     send_plaintext_hello(protocol)
-    send_plaintext_connect_response(protocol, False)
+    send_plaintext_auth_response(protocol, False)
 
     await connect_task
     ping_request_bytes = [b"\x00", b"\x00", b"\x07"]
@@ -1049,7 +1033,7 @@ async def test_respond_to_ping_request(
     conn, transport, protocol, connect_task = plaintext_connect_task_with_login
 
     send_plaintext_hello(protocol)
-    send_plaintext_connect_response(protocol, False)
+    send_plaintext_auth_response(protocol, False)
 
     await connect_task
     assert conn.is_connected
@@ -1069,7 +1053,7 @@ async def test_unknown_protobuf_message_type_logged(
     caplog: pytest.LogCaptureFixture,
 ) -> None:
     """Test unknown protobuf messages are logged but do not cause the connection to collapse."""
-    client, connection, transport, protocol = api_client
+    client, connection, _transport, protocol = api_client
     response: message.Message = DeviceInfoResponse(
         name="realname",
         friendly_name="My Device",
@@ -1100,7 +1084,7 @@ async def test_bad_protobuf_message_drops_connection(
     caplog: pytest.LogCaptureFixture,
 ) -> None:
     """Test ad bad protobuf messages is logged and causes the connection to collapse."""
-    client, connection, transport, protocol = api_client
+    client, connection, _transport, protocol = api_client
     msg: message.Message = TextSensorStateResponse(
         key=1, state="invalid", missing_state=False
     )
@@ -1128,9 +1112,9 @@ async def test_connection_cannot_be_reused(
     ],
 ) -> None:
     """Test that we raise when trying to connect when already connected."""
-    conn, transport, protocol, connect_task = plaintext_connect_task_with_login
+    conn, _transport, protocol, connect_task = plaintext_connect_task_with_login
     send_plaintext_hello(protocol)
-    send_plaintext_connect_response(protocol, False)
+    send_plaintext_auth_response(protocol, False)
     await connect_task
     with pytest.raises(RuntimeError):
         await conn.start_resolve_host()
@@ -1214,7 +1198,7 @@ async def test_internal_message_received_immediately_after_connection(
 
         # Now send the expected hello response to let connection complete
         send_plaintext_hello(protocol)
-        send_plaintext_connect_response(protocol, False)
+        send_plaintext_auth_response(protocol, False)
 
         # Wait for the connect task to complete
         await connect_task
@@ -1291,3 +1275,222 @@ async def test_report_fatal_error_with_log_errors_false(
 
     # Verify the error is still stored internally
     assert conn._fatal_exception is regular_error
+
+
+def test_send_messages_after_cleanup_raises_exception(conn: APIConnection) -> None:
+    """Test that calling send_messages after cleanup raises an exception instead of segfaulting."""
+    # Set up the connection state as if it was connected
+    conn.connection_state = ConnectionState.CONNECTED
+    conn._handshake_complete = True
+
+    # Report a fatal error to trigger cleanup (which sets _frame_helper to None)
+    conn.report_fatal_error(APIConnectionError("Connection lost"))
+
+    # Now try to send messages - this should raise instead of segfaulting
+    # report_fatal_error clears _handshake_complete, so we get a different error
+    with pytest.raises(
+        ConnectionNotEstablishedAPIError, match="Connection isn't established yet"
+    ):
+        conn.send_messages((PingRequest(),))
+
+
+def test_send_messages_with_fatal_exception_and_no_frame_helper(
+    conn: APIConnection,
+) -> None:
+    """Test that send_messages raises the fatal exception when frame_helper is None and fatal exception is set."""
+    # Set up the connection state as if it was connected
+    conn.connection_state = ConnectionState.CONNECTED
+    conn._handshake_complete = True
+
+    # Report an auth error to trigger cleanup and set fatal exception
+    fatal_error = InvalidAuthAPIError("Authentication failed")
+    conn.report_fatal_error(fatal_error)
+
+    # Try to send messages - should raise ConnectionNotEstablishedAPIError
+    # because report_fatal_error clears _handshake_complete
+    with pytest.raises(
+        ConnectionNotEstablishedAPIError, match="Connection isn't established yet"
+    ):
+        conn.send_messages((PingRequest(),))
+
+
+def test_send_messages_after_report_fatal_error(conn: APIConnection) -> None:
+    """Test that send_messages raises exception after report_fatal_error is called."""
+    # Set up the connection state as if it was connected
+    conn.connection_state = ConnectionState.CONNECTED
+    conn._handshake_complete = True
+
+    # Report a fatal error
+    fatal_error = APIConnectionError("Connection lost")
+    conn.report_fatal_error(fatal_error)
+
+    # Verify fatal exception was set
+    assert conn._fatal_exception is fatal_error
+
+    # Try to send messages - should raise ConnectionNotEstablishedAPIError
+    # because report_fatal_error clears _handshake_complete
+    with pytest.raises(
+        ConnectionNotEstablishedAPIError, match="Connection isn't established yet"
+    ):
+        conn.send_messages((PingRequest(),))
+
+
+async def test_disconnect_skips_sending_when_frame_helper_is_none(
+    conn: APIConnection,
+) -> None:
+    """Test that disconnect() doesn't try to send messages when frame_helper is None."""
+    # Set up the connection state as if it was connected
+    conn.connection_state = ConnectionState.CONNECTED
+    conn._handshake_complete = True
+
+    # Report a fatal error to trigger cleanup (sets frame_helper to None)
+    conn.report_fatal_error(APIConnectionError("Test error"))
+
+    # Call disconnect - should not raise even though frame_helper is None
+    await conn.disconnect()
+
+    # Verify cleanup was called
+    assert conn.connection_state == ConnectionState.CLOSED
+
+
+def test_force_disconnect_skips_sending_when_frame_helper_is_none(
+    conn: APIConnection,
+) -> None:
+    """Test that force_disconnect() doesn't try to send messages when frame_helper is None."""
+    # Set up the connection state as if it was connected
+    conn.connection_state = ConnectionState.CONNECTED
+    conn._handshake_complete = True
+
+    # Report a fatal error to trigger cleanup (sets frame_helper to None)
+    conn.report_fatal_error(APIConnectionError("Test error"))
+
+    # Call force_disconnect - should not raise even though frame_helper is None
+    conn.force_disconnect()
+
+    # Verify cleanup was called
+    assert conn.connection_state == ConnectionState.CLOSED
+
+
+def test_send_messages_race_condition_with_cleanup(conn: APIConnection) -> None:
+    """Test handling of race condition where cleanup happens during send_messages.
+
+    This test verifies that the segfault fix prevents crashes when _frame_helper
+    becomes None between the handshake check and its usage.
+    """
+    # Set up the connection state as if it was connected
+    conn.connection_state = ConnectionState.CONNECTED
+    conn._handshake_complete = True
+
+    # Report a fatal error to trigger cleanup (sets frame_helper to None)
+    conn.report_fatal_error(APIConnectionError("Simulated race condition"))
+
+    # This would previously segfault if frame_helper became None during operation
+    # Now it should handle it gracefully by raising ConnectionNotEstablishedAPIError
+    # because report_fatal_error clears _handshake_complete
+    with pytest.raises(
+        ConnectionNotEstablishedAPIError, match="Connection isn't established yet"
+    ):
+        conn.send_messages((PingRequest(),))
+
+    # Verify connection is closed
+    assert conn.connection_state == ConnectionState.CLOSED
+
+
+def test_send_messages_when_frame_helper_is_none_with_handshake_complete(
+    conn: APIConnection,
+) -> None:
+    """Test send_messages when frame_helper is None but handshake is complete.
+
+    This covers the edge case where _handshake_complete is True but _frame_helper
+    is None, which triggers the segfault protection that raises the fatal exception.
+    """
+    # Set up connection state as connected with handshake complete
+    conn.connection_state = ConnectionState.CONNECTED
+    conn._handshake_complete = True
+
+    # Report a fatal error to trigger cleanup (sets _frame_helper to None)
+    conn.report_fatal_error(APIConnectionError("Test error"))
+
+    # Now manually set state back to simulate a race condition
+    # where these get set after cleanup but frame_helper is still None
+    conn._handshake_complete = True
+    conn.connection_state = ConnectionState.CONNECTED
+
+    # Should raise the fatal exception since _frame_helper is None
+    # This tests the segfault protection that raises _fatal_exception when available
+    with pytest.raises(APIConnectionError, match="Test error"):
+        conn.send_messages((PingRequest(),))
+
+
+def test_send_messages_when_frame_helper_none_no_fatal_exception(
+    conn: APIConnection,
+) -> None:
+    """Test send_messages when frame_helper is None and no fatal exception.
+
+    This covers the fallback case where ConnectionNotEstablishedAPIError
+    is raised when _frame_helper is None but _fatal_exception is also None.
+    """
+    # Set up connection state as connected with handshake complete
+    conn.connection_state = ConnectionState.CONNECTED
+    conn._handshake_complete = True
+
+    # Report a fatal error to trigger cleanup (sets _frame_helper to None)
+    conn.report_fatal_error(APIConnectionError("Test error"))
+
+    # Clear the fatal exception and reset state to simulate a race condition
+    conn._fatal_exception = None
+    conn._handshake_complete = True
+    conn.connection_state = ConnectionState.CONNECTED
+
+    # Should raise ConnectionNotEstablishedAPIError since both
+    # _frame_helper and _fatal_exception are None
+    with pytest.raises(ConnectionNotEstablishedAPIError, match="Connection is closed"):
+        conn.send_messages((PingRequest(),))
+
+
+def test_cleanup_debug_message_with_error(
+    conn: APIConnection,
+    caplog: pytest.LogCaptureFixture,
+) -> None:
+    """Test that cleanup debug message includes error when there is a fatal exception."""
+    # Enable debug logging
+    caplog.set_level(logging.DEBUG)
+    conn._debug_enabled = True
+    conn.log_name = "test-device"
+
+    # Set connection to a non-closed state so cleanup will run
+    conn.connection_state = ConnectionState.CONNECTED
+
+    # Set a fatal exception directly and trigger cleanup
+    conn._fatal_exception = APIConnectionError("Test error occurred")
+
+    # Clear logs and call force_disconnect which will trigger cleanup
+    caplog.clear()
+    conn.force_disconnect()
+
+    # Check that the error is included in the debug message
+    assert (
+        "Cleaning up connection to test-device (error: Test error occurred)"
+        in caplog.text
+    )
+
+
+def test_cleanup_debug_message_without_error(
+    conn: APIConnection,
+    caplog: pytest.LogCaptureFixture,
+) -> None:
+    """Test that cleanup debug message doesn't include error when there is no fatal exception."""
+    # Enable debug logging
+    caplog.set_level(logging.DEBUG)
+    conn._debug_enabled = True
+    conn.log_name = "test-device"
+
+    # Set connection to a non-closed state so cleanup will run
+    conn.connection_state = ConnectionState.CONNECTED
+
+    # Test cleanup without fatal exception via force_disconnect
+    caplog.clear()
+    conn.force_disconnect()
+    # Should not contain "(error:" part
+    assert "Cleaning up connection to test-device" in caplog.text
+    assert "(error:" not in caplog.text
